function w = lfdm(prob_train, tag_train, v, Q)

% Input
% prob_train: probability matrix, 
%               the first dimension is number of training sample
%               the second dimension is the dimension of feature vector
%               the third dimension is number of features
%               the fourth dimension is number of classes
% Output
% w: weight matrix 

% Please cite our paper, if you use this code for your project. Thanks.
% A J Ma, P C Yuen and J H Lai, "Linear dependency modeling for classifier fusion and feature combination",
% IEEE Trans. Pattern Analysis and Machine Intelligence, vol. 35, no. 5, pp. 1135-1148, 2013.


[num_train dim_fea num_feas num_class] = size(prob_train);
num_classifier = dim_fea * num_feas;
prob_train = reshape(prob_train, [num_train num_classifier num_class]);

C = 1 / v / num_train;
f = [-1 C*ones(1,num_train) zeros(1,num_classifier*num_class)]; % For efficiency, set the same \xi for each sample

A = zeros(num_train*(num_class-1), 1+num_train+num_classifier*num_class);
A(:,1) = 1;
n = 1;
for j = 1:num_train
    yl = tag_train(j);
    tmp = prob_train(j,:,yl);
    tmp = tmp(:)';
    A(n:n+num_class-2,j+1) = -1;
    A(n:n+num_class-2,1+num_train+(yl-1)*num_classifier+1:1+num_train+yl*num_classifier) ...
        = - repmat(tmp,[num_class-1 1]);
    for yl2 = 1:num_class
        if yl2 == yl
            continue
        end
        tmp = prob_train(j,:,yl2);
        tmp = tmp(:)';
        A(n,1+num_train+(yl2-1)*num_classifier+1:1+num_train+yl2*num_classifier) = tmp;
        n = n + 1;
    end
end

b = zeros(num_train * (num_class-1),1);
Aeq = zeros(num_class, num_classifier*num_class + num_train + 1);
for ii = 1:num_class
    Aeq(ii,1+num_train+num_classifier*(ii-1)+1:1+num_train+num_classifier*ii) = 1;
end
beq = ones(num_class,1) * Q;
lb = zeros(num_classifier*num_class + num_train + 1,1);
ub = ones(num_classifier*num_class + num_train + 1,1) * 2;
ub(1:1+num_train) = inf;

[a cost] = linprog(f,A,b,Aeq,beq,lb,ub);
w = reshape(a(2+num_train:end), [dim_fea num_feas num_class]);